#include "lexeme.h"

#define _CRT_SECURE_NO_WARNINGS
#include <string.h>

#include "scm-mem.h"

// -------------------------------------------------------------------------------------------- //
// * Implementation * //
// -------------------------------------------------------------------------------------------- //

scm::Lexeme::Lexeme()
{
	token = NULL;
	type = IS_INVALID;
	tag = 0;
}

scm::Lexeme::~Lexeme()
{
	clear();
}

scm::Lexeme::Lexeme(const Lexeme& lexeme)
{
	if (lexeme.token == NULL) token = NULL;
	else
	{
		token = new char[strlen(lexeme.token) + 1];
		strcpy(token, lexeme.token);
	}

	type = lexeme.type;
	tag = lexeme.tag;
}
// -------------------------------------------------------------------------------------------- //

// * swap & assignment * //
// -------------------------------------------------------------------------------------------- //
const scm::Lexeme& scm::Lexeme::operator=(Lexeme lexeme)
{
	swap(lexeme);
	return (*this);
}

void scm::Lexeme::swap(Lexeme& lexeme)
{
	scm::swap(token, lexeme.token);
	scm::swap(type, lexeme.type);
	scm::swap(tag, lexeme.tag);
}
// -------------------------------------------------------------------------------------------- //

// * set calls * //
// -------------------------------------------------------------------------------------------- //
void scm::Lexeme::set(const char* _token, const TYPE _type, const int _tag)
{
	clear();
	if ((_token == NULL) || (_type == IS_INVALID)) return;

	token = new char[strlen(_token) + 1];
	strcpy(token, _token);

	type = _type;
	tag = _tag;
}

void scm::Lexeme::set(const char* _token, const TYPE _type)
{
	set(_token, _type, 0);
}
// -------------------------------------------------------------------------------------------- //

// * get calls * //
// -------------------------------------------------------------------------------------------- //
const char* scm::Lexeme::get_token() const
{
	return token;
}

scm::Lexeme::TYPE scm::Lexeme::get_type() const
{
	return type;
}

int scm::Lexeme::get_tag() const
{
	return tag;
}
// -------------------------------------------------------------------------------------------- //

// * clear * //
// -------------------------------------------------------------------------------------------- //
void scm::Lexeme::clear()
{
	if (token != NULL) delete[] token;

	token = NULL;
	type = IS_INVALID;
	tag = 0;
}
// -------------------------------------------------------------------------------------------- //

// * check lexeme type * //
// -------------------------------------------------------------------------------------------- //
bool scm::Lexeme::is_operator() const
{
	return is_operator(type);
}

bool scm::Lexeme::is_function() const
{
	return is_function(type);
}

bool scm::Lexeme::is_ctor() const
{
	return is_ctor(type);
}

bool scm::Lexeme::is_value() const
{
	return is_value(type);
}

bool scm::Lexeme::is_operator(const Lexeme::TYPE type)
{
	return (
		(type == IS_OP_ADD) || (type == IS_OP_SUB) ||
		(type == IS_OP_MUL) || (type == IS_OP_DIV) || (type == IS_OP_MOD) ||
		(type == IS_OP_PLUS) || (type == IS_OP_MINUS) ||
		(type == IS_OP_EXP) ||
		(type == IS_OP_EQ) || (type == IS_OP_NEQ) ||
		(type == IS_OP_LOGICAL_AND)  || (type == IS_OP_LOGICAL_OR) ||
		(type == IS_OP_LT) || (type == IS_OP_GT) ||
		(type == IS_OP_LEQ) || (type == IS_OP_GEQ)
		);
}

bool scm::Lexeme::is_function(const Lexeme::TYPE type)
{
	return (
		(type == IS_SIN_FUNCTION) || (type == IS_COS_FUNCTION) ||
		(type == IS_TAN_FUNCTION) || (type == IS_LOG_FUNCTION) ||
		(type == IS_SQRT_FUNCTION) ||
		(type == IS_MIN_FUNCTION) || (type == IS_MAX_FUNCTION) ||
		(type == IS_VECMIN_FUNCTION) || (type == IS_VECMAX_FUNCTION) ||
		(type == IS_ABS_FUNCTION) || (type == IS_UNIRAND_FUNCTION) ||
		(type == IS_DOT_PRODUCT_FUNCTION) ||
		(type == IS_L2NORM_FUNCTION) || (type == IS_CNORM_FUNCTION) ||
		(type == IS_TO_STRING_FUNCTION) ||
		(type == IS_SIZE_FUNCTION) || (type == IS_DEFINED_FUNCTION)
		);
}

bool scm::Lexeme::is_ctor(const Lexeme::TYPE type)
{
	return (
		(type == IS_VECTOR_CTOR) || 
		(type == IS_VECTOR_CONST_CTOR) ||
		(type == IS_VECTOR_UNIRAND_CTOR))
		;
}

bool scm::Lexeme::is_value(const Lexeme::TYPE type)
{
	return (
		(type == IS_INTEGER) || (type == IS_FLOAT) ||
		(type == IS_BOOL) ||
		(type == IS_STRING)
		);
}
// -------------------------------------------------------------------------------------------- //
