module sfx_sheba_noit_param
    !< @brief SHEBA surface flux model parameters
    !< @details  all in SI units

    ! modules used
    ! --------------------------------------------------------------------------------
    use sfx_phys_const
    ! --------------------------------------------------------------------------------

    ! directives list
    ! --------------------------------------------------------------------------------
    implicit none
    ! --------------------------------------------------------------------------------


    !< von Karman constant [n/d]
    real, parameter :: kappa = 0.40
    !< inverse Prandtl (turbulent) number in neutral conditions [n/d]
    real, parameter :: Pr_t_0_inv = 1.15
    !< inverse Prandtl (turbulent) number in free convection [n/d]
    real, parameter :: Pr_t_inf_inv = 3.5

    real, parameter :: Rib_max = 0.5

    !< stability function coeff. (unstable)
    real, parameter :: alpha_m = 16.0
    real, parameter :: alpha_h = 16.0
    real, parameter :: alpha_h_fix = 16.0

    !< stability function coeff. (stable)
    real, parameter :: a_m = 5.0
    real, parameter :: b_m = a_m / 6.5
    real, parameter :: a_h = 5.0
    real, parameter :: b_h = 5.0
    real, parameter :: c_h = 3.0

    real, parameter  ::  gamma = 2.91, zeta_a = 3.6 ! for stable psi


end module sfx_sheba_noit_param
